#!/bin/bash
#set -x
set -e

echoPrefix="\033[36m+"
echoSuffix="\033[0m"

codePath=/home/www/deploy/wms8_ztf_2
serverUrl=http://127.0.0.1:8094

deploy() {
  # 更新代码
  echo -e "$echoPrefix cd $codePath/wms-ui-mobile $echoSuffix"
  cd $codePath/wms-ui-mobile
  echo -e "$echoPrefix git checkout . $echoSuffix"
  git checkout .
  echo -e "$echoPrefix git pull $echoSuffix"
  git pull
  # 版本号
  major='1'
  minor=$(date +%Y%m%d)
  patch=$(date +1%H%M%S | sed -e 's/^[[:space:]]*//')
  version="$major.$minor.$patch"
  echo -e "$echoPrefix 'export const version = '$version'' > src/version.ts $echoSuffix"
  echo "export const version = '$version'" > src/version.ts

  echo "###--代码更新完成--------------------------------------------------------------------------###"
  echo ""

  # 编译
  echo -e "$echoPrefix yarn run build $echoSuffix"
  yarn run build
  echo "###--编译完成-----------------------------------------------------------------------------###"
  echo ""

  # 创建打包文件夹
  echo -e "$echoPrefix rm -rf ./tmp $echoSuffix"
  rm -rf ./tmp

  echo -e "$echoPrefix mkdir -p ./tmp/web/wms_mobile $echoSuffix"
  mkdir -p ./tmp/web/wms_mobile

  echo -e "$echoPrefix cp -r ./dist/* ./tmp/web/wms_mobile/ $echoSuffix"
  cp -r ./dist/* ./tmp/web/wms_mobile/

  echo -e "$echoPrefix cd ./tmp $echoSuffix"
  cd ./tmp
  echo -e "$echoPrefix zip -r web.zip web $echoSuffix"
  zip -r web.zip web

  echo "###--打包完成-----------------------------------------------------------------------------###"
  echo ""

  # 上传文件
  echo -e "$echoPrefix curl -X POST -F 'file=@$codePath/wms-ui-mobile/tmp/web.zip' $serverUrl/wms_api/api/pages/sys/pdaVersion/PdaPackUpload@uploadFile $echoSuffix"
  res_json=$(curl -X POST -F 'file=@'$codePath'/wms-ui-mobile/tmp/web.zip' -H "deploy: VPitn9zuOizfciiqY53vtuVz9FXCQ0Ddx3tEWMSZIbDDIbPtEJ3nbFnJOfXKbxdT875sRTv1hWUNkRpBsS1pciHB3h8YcI5huX5ViDnO7XAD0uaiSlZJaUW6aym" $serverUrl/wms_api/api/pages/sys/pdaVersion/PdaPackUpload@uploadFile)
  filePath=$(echo $res_json | awk -F '"' '{print $34}')
  echo "filePath = $filePath"

  req_json='{"args":[{"app_code":"ZTF_WEB","app_name":null,"major":"'$major'","minor":"'$minor'","patch":"'$patch'","file_path":"'$filePath'"}]}'
  res_json=$(curl -H "Content-type: application/json" -H "deploy: VPitn9zuOizfciiqY53vtuVz9FXCQ0Ddx3tEWMSZIbDDIbPtEJ3nbFnJOfXKbxdT875sRTv1hWUNkRpBsS1pciHB3h8YcI5huX5ViDnO7XAD0uaiSlZJaUW6aym" -X POST -d "$req_json" $serverUrl/wms_api/api/pages/sys/pdaVersion/PdaPackUpload@savePdaVersion)
  echo -e "$echoPrefix $res_json $echoSuffix"
  echo ""

  # 还原文件
  # echo -e "$echoPrefix rm -rf ./tmp $echoSuffix"
  # rm -rf ./tmp
  echo -e "$echoPrefix cd $codePath $echoSuffix"
  cd $codePath
  echo -e "$echoPrefix git checkout . $echoSuffix"
  git checkout .
  echo -e "$echoPrefix 版本号信息: version=$version, major=$major, minor=$minor, patch=$patch $echoSuffix"
  echo ""
}

cmd=$1
if [ "$cmd" == "deploy" ];then
  deploy
else
  echo "使用 deploy 命名打包pda"
fi

# 中通服 PDA
# bash <(curl -s http://git.yvanui.com/lizhiwei/jztd-deploy/raw/master/hua_wei_yun/09wms-ztf-pad-test.sh) [cmd]
